/*
** Module   : FillInWidget.java
** Abstract : server side FillIn widget implementation
**
** Copyright (c) 2005-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------Description----------------------
** 001 NVS 20051026   @23490 Created initial version.
** 002 GES 20060114   @23909 Removed totally useless screen-value attr.
** 003 SIY 20060404   @25357 Support for the text group.
** 004 GES 20060803   @28375 Cleanup and refactoring.
** 005 GES 20080320   @37590 Added TYPE attribute.
** 006 SVL 20101006          Added NATIVE attribute.
** 007 SIY 20111101          Updated imports.
** 008 CA  20130221          Refactored usage of TYPE attribute.
** 009 CA  20130927          Added LegacyResource annotation, to determine resource type.
** 010 CA  20131115          Changes related to properly instantiate dynamic resources.
** 011 CA  20140926          Added shared widget configuration support, to allow GUI/ChUI
**                           concrete implementation for the same widget ID.  Refactored the
**                           widget configuration classes: all fields were made public; these
**                           classes need to be as dumb as possible, all logic related to
**                           setting/getting a certain field should be at the widget or at the
**                           caller. Refs #2254
** 012 SVL 20141102          Make this class implement AutoZapElement.
** 013 GES 20141126          Added SET-SELECTION() method.
** 014 IAS 20150604          Added support for the DISABLE-AUTO-ZAP and SELECTED-TEXT-related 
**                           attribute/methods.
*/ 

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;
import com.goldencode.util.*;

/**
 * Server side FillIn widget implementation.
 */
@LegacyResource(resource = LegacyResource.FILL_IN)
public class FillInWidget
extends ControlTextWidget<FillInConfig>
implements AutoZapElement, 
           SelectableText
{
   /**
    * Default constructor.
    */
   public FillInWidget()
   {
      this(false);
   }
   
   /**
    * Default constructor.
    * 
    * @param   dynamic
    *          Flag indicating if this is a static or dynamic resource.
    */
   public FillInWidget(boolean dynamic)
   {
      super(dynamic, new FillInConfig());
   }

   /**
    * Sets the AUTO-RETURN option.
    * 
    * @param    autoReturn
    *           <code>true</code> if the option is set to ON.
    */
   public void setAutoReturn(boolean autoReturn)
   {
      config.autoReturn = autoReturn;
      pushScreenDefinition();
   }

   /**
    * Sets the DEBLANK option.
    * 
    * @param    deblank
    *           <code>true</code> if the option is set to ON.
    */
   public void setDeblank(boolean deblank)
   {
      // defined only for fill-ins
      config.deblank = deblank;
      pushScreenDefinition();
   }

   /**
    * Sets a read only state.
    * 
    * @param   newReadOnly
    *          Is widget read only.
    */
   public void setReadOnly(boolean newReadOnly)
   {
      config.readOnly = newReadOnly;
   }

   /**
    * Start text group of widgets (word wrap mode).
    * 
    * @param   prev
    *          Reference to previous fill-in widget or null if no such
    *          widgets.
    */
   public void startGrouping(FillInWidget prev)
   {
      config.grouping = true;
      
      if (prev != null)
      {
         prev.setNextGroupItem(this);
         config.prevId = prev.getId();
      }
   }

   /**
    * Link widget with the next widget in the text group.
    * 
    * @param   next
    *          Reference to next widget in group.
    */
   public void setNextGroupItem(FillInWidget next)
   {
      if (next != null)
         config.nextId = next.getId();
   }

   /**
    * Reset grouping flag and links to previous and next widget in the text
    * group.
    */
   public void resetGrouping()
   {
      config.grouping = false;
      config.nextId = -1;
      config.prevId = -1;
   }

   /**
    * Set value of the NATIVE attribute.
    *
    * @param   nativ
    *          New value for the attribute.
    */
   public void setNative(boolean nativ)
   {
      config.nativ = nativ;
   }

   /**
    * Selects text in a widget between start and end points.
    * 
    * @param    start
    *           Starting offset.
    * @param    end
    *           Ending offset.
    *
    * @return   <code>true</code> if the operation succeeded.
    */
   @Override
   public logical setSelection(int64 start, int64 end)
   {
      return setSelection(start == null || start.isUnknown() ? 0 : start.intValue(), 
            end == null || end.isUnknown() ? 0 : end.intValue());
   }
   
   /**
    * Selects text in a widget between start and end points.
    * 
    * @param    start
    *           Starting offset.
    * @param    end
    *           Ending offset.
    *
    * @return   <code>true</code> if the operation succeeded.
    */
   @Override
   public logical setSelection(int64 start, int end)
   {
      return setSelection(start == null || start.isUnknown() ? 0 : start.intValue(), end);
   }
   
   /**
    * Selects text in a widget between start and end points.
    * 
    * @param    start
    *           Starting offset.
    * @param    end
    *           Ending offset.
    *
    * @return   <code>true</code> if the operation succeeded.
    */
   @Override
   public logical setSelection(int start, int64 end)
   {
      return setSelection(start, end == null || end.isUnknown() ? 0 : end.intValue());
   }
   
   /**
    * Selects text in a widget between start and end points.
    * 
    * @param    start
    *           Starting offset.
    * @param    end
    *           Ending offset.
    *
    * @return   <code>true</code> if the operation succeeded.
    */
   @Override
   public logical setSelection(int start, int end)
   {
      if (!canAccess("SET-SELECTION"))
      {
         return new logical();
      }

      if (start == 0 || end == 0)
      {
         ErrorManager.recordOrShowError(4057, 
               String.format(
               "Attribute SET-SELECTION for the %s %s has an invalid value of 0", 
               type(), widgetName()),
               false);
         return new logical();
      }
      int s1 = Math.min(start, end);
      int s2 = Math.max(start,  end);
      if (s1 < 0 || s2 > config.widthChars)
      {
         return new logical(false);
      }

      config.selectionStart = s1;
      config.selectionEnd = s2;
      config.selectionActive = true;
      pushScreenDefinition();
      return new logical(true);
   }
   
   /**
    * Get SELECTION-TEXT attribute.
    *
    * @return  SELECTION-TEXT value   
    */
   public character getSelectionText()
   {
      if (!canAccess("SELECTION-TEXT"))
      {
         return new character();
      }

      if (!config.selectionActive)
      {
         return new character("");
      }

      if (!config.selectionActive)
      {
         return new character();
      }
      character sv = getScreenValue();
      String s = sv == null || sv.isUnknown() ? "" : sv.toStringMessage();
      s += StringHelper.repeatChar(' ', (int)config.widthChars - s.length());
      s = config.selectionEnd + 1 >= config.widthChars ?
            s.substring(config.selectionStart) :
            s.substring(config.selectionStart, config.selectionEnd + 1);
            
      return new character(s);
   }

   /**
    * Get SELECTION-END attribute.
    *
    * @return  SELECTION-END value   
    */
   public integer getSelectionEnd()
   {
      if (!canAccess("SELECTION-END"))
      {
         return new integer();
      }

      if (!config.selectionActive)
      {
         return new integer();
      }
      return new integer(config.selectionEnd);
   }

   /**
    * Get SELECTION-START attribute.
    *
    * @return  SELECTION-START value   
    */
   public integer getSelectionStart()
   {
      if (!canAccess("SELECTION-START"))
      {
         return new integer();
      }

      if (!config.selectionActive)
      {
         return new integer();
      }
      return new integer(config.selectionStart);
   }

   /**
    * Get TEXT-SELECTED attribute.
    *
    * @return  TEXT-SELECTED value   
    */
   @Override
   public logical isTextSelected()
   {
      if (!canAccess("TEXT-SELECTION"))
      {
         return new logical();
      }
      return new logical(config.selectionActive);
   }

   /**
    * Clears text selection
    */
   public logical clearSelection()
   {
      boolean wasSelection = config.selectionActive;
      config.selectionActive = false;
      pushScreenDefinition();
      return new logical(wasSelection);
   }
   
   /**
    * Sets the DISABLE-AUTO-ZAP option.
    * 
    * @param    disableAutoZap
    *           <code>true</code> if the option is set to ON.
    */
   public void setDisableAutoZap(boolean disableAutoZap)
   {
      // TODO: should update config.disableAutoZap
      config.autoZap = !disableAutoZap;
      pushScreenDefinition();
   }

   /**
    * Set DISABLE-AUTO-ZAP attribute. 
    * 
    * @param    value
    *           New value for the attribute.
    */
   @Override
   public void setDisableAutoZap(logical value)
   {
      setDisableAutoZap(value == null || value.isUnknown() ? false : value.booleanValue());
   }

   /**
    * Get value of the DISABLE-AUTO-ZAP attribute.
    *
    * @return   value of the attribute.
    */
   @Override
   public logical isDisableAutoZap()
   {
      return new logical(config.disableAutoZap);
   }

   /**
    * Set AUTO-ZAP attribute.
    * 
    * @param    value
    *           New value for the attribute.
    */
   public void setAutoZap(boolean value)
   {
      config.autoZap = value;
      pushScreenDefinition();
   }

   /**
    * Set value of the AUTO-ZAP attribute.
    *
    * @param    autoZap
    *           Value of the attribute.
    */
   public void setAutoZap(logical autoZap)
   {
      if (autoZap.isUnknown())
         return;

      setAutoZap(autoZap.booleanValue());
   }

   /**
    * Get value of the AUTO-ZAP attribute.
    *
    * @return   value of the attribute.
    */
   public logical isAutoZap()
   {
      return new logical(config.autoZap);
   }
}
